@@ -26,16 +26,12 @@ BEGIN_EVENT_TABLE ( MyPropertySheet, wxPropertySheetDialog )
 	EVT_BUTTON (
 	    APPLICATION_DIR_BROWSE,
 	    MyPropertySheet::OnApplicationDirBrowse )
-	EVT_BUTTON (
-	    BROWSER_COMMAND_BROWSE,
-	    MyPropertySheet::OnBrowserCommandBrowse )
 END_EVENT_TABLE()
 
 MyPropertySheet::MyPropertySheet (
     wxWindow *parent,
     XmlCtrlProperties& propertiesParameter,
     wxString& applicationDirParameter,
-    wxString& browserCommandParameter,
     bool rememberOpenTabsParameter,
     bool libxmlNetAccessParameter,
     bool singleInstanceCheckParameter,
@@ -45,15 +41,16 @@ MyPropertySheet::MyPropertySheet (
     bool expandInternalEntitiesParameter,
     bool showFullPathOnFrameParameter,
     int lang,
+    const std::set<const wxLanguageInfo *> &translations,
     wxWindowID id,
     wxString title,
     const wxPoint& position,
     const wxSize& size,
-    long style ) : wxPropertySheetDialog (
-		        parent, id, title, position, size, style ),
-		properties ( propertiesParameter ),
-		applicationDir ( applicationDirParameter ),
-		browserCommand ( browserCommandParameter )
+    long style
+    )
+	: wxPropertySheetDialog ( parent, id, title, position, size, style )
+	, properties ( propertiesParameter )
+	, applicationDir ( applicationDirParameter )
 {
 	CreateButtons ( wxOK | wxCANCEL );
 
@@ -140,7 +137,6 @@ MyPropertySheet::MyPropertySheet (
 	wxPanel *generalPanel = new wxPanel ( GetBookCtrl() );
 	wxBoxSizer *vsizer = new wxBoxSizer ( wxVERTICAL );
 	wxBoxSizer *hsizer = new wxBoxSizer ( wxHORIZONTAL );
-	wxBoxSizer *hsizerBrowser = new wxBoxSizer ( wxHORIZONTAL );
 	wxBoxSizer *hsizerCheckboxes = new wxBoxSizer ( wxHORIZONTAL );
 	wxBoxSizer *vsizerCheckbox1 = new wxBoxSizer ( wxVERTICAL );
 	wxBoxSizer *vsizerCheckbox2 = new wxBoxSizer ( wxVERTICAL );
@@ -161,22 +157,6 @@ MyPropertySheet::MyPropertySheet (
 	    _ ( "Browse" ),
 	    wxDefaultPosition,
 	    wxSize ( -1, applicationDirEdit->GetSize().GetHeight() ) );
-	wxStaticText *labelBrowser = new wxStaticText (
-	    generalPanel,
-	    wxID_ANY,
-	    _ ( "Browser" ) );
-	browserCommandEdit = new wxTextCtrl (
-	    generalPanel,
-	    wxID_ANY,
-	    browserCommand,
-	    wxDefaultPosition,
-	    wxSize ( 240, -1 ) );
-	wxButton *browseCommand = new wxButton (
-	    generalPanel,
-	    BROWSER_COMMAND_BROWSE,
-	    _ ( "Browse" ),
-	    wxDefaultPosition,
-	    wxSize ( -1, browserCommandEdit->GetSize().GetHeight() ) );
 
 	wxStaticText *labelLanguage = new wxStaticText (
 	    generalPanel,
@@ -185,62 +165,19 @@ MyPropertySheet::MyPropertySheet (
 	languageBox = new wxChoice (
 	    generalPanel,
 	    wxID_ANY );
+	languageBox->SetExtraStyle ( languageBox->GetExtraStyle() | wxCB_SORT );
 
-    languageBox->Insert ( _T ( "Catalan" ), INDEX_CATALAN );
-	languageBox->Insert ( _T ( "Chinese Simplified" ), INDEX_CHINESE_SIMPLIFIED );
-	languageBox->Insert ( _T ( "Chinese Traditional" ), INDEX_CHINESE_TRADITIONAL );
-	languageBox->Insert ( _T ( "Dutch" ), INDEX_DUTCH );
-	languageBox->Insert ( _T ( "English (US)" ), INDEX_ENGLISH_US );
-	languageBox->Insert ( _T ( "French" ), INDEX_FRENCH );
-	languageBox->Insert ( _T ( "German" ), INDEX_GERMAN );
-	languageBox->Insert ( _T ( "Italian" ), INDEX_ITALIAN );
-	languageBox->Insert ( _T ( "Russian" ), INDEX_RUSSIAN );
-	languageBox->Insert ( _T ( "Slovak" ), INDEX_SLOVAK );
-	languageBox->Insert ( _T ( "Spanish" ), INDEX_SPANISH );
-	languageBox->Insert ( _T ( "Swedish" ), INDEX_SWEDISH );
-	languageBox->Insert ( _T ( "Ukrainian" ), INDEX_UKRAINIAN );
+	languageBox->Append ( _ ( "Default" ), ( void* ) wxLANGUAGE_ENGLISH_US );
+	languageBox->SetSelection ( 0 );
 
-	switch ( lang )
+	int index;
+	std::set<const wxLanguageInfo *>::const_iterator t = translations.begin();
+	for ( ; t != translations.end(); t++ )
 	{
-        case wxLANGUAGE_CATALAN:
-            languageBox->SetSelection ( INDEX_CATALAN );
-            break;
-		case wxLANGUAGE_GERMAN:
-			languageBox->SetSelection ( INDEX_GERMAN );
-			break;
-		case wxLANGUAGE_DUTCH:
-            languageBox->SetSelection ( INDEX_DUTCH );
-            break;
-		case wxLANGUAGE_FRENCH:
-			languageBox->SetSelection ( INDEX_FRENCH );
-			break;
-		case wxLANGUAGE_ITALIAN:
-			languageBox->SetSelection ( INDEX_ITALIAN );
-			break;
-		case wxLANGUAGE_SLOVAK:
-			languageBox->SetSelection ( INDEX_SLOVAK );
-			break;
-		case wxLANGUAGE_SWEDISH:
-			languageBox->SetSelection ( INDEX_SWEDISH );
-			break;
-		case wxLANGUAGE_RUSSIAN:
-			languageBox->SetSelection ( INDEX_RUSSIAN );
-			break;
-		case wxLANGUAGE_CHINESE_SIMPLIFIED:
-			languageBox->SetSelection ( INDEX_CHINESE_SIMPLIFIED );
-			break;
-		case wxLANGUAGE_CHINESE_TRADITIONAL:
-			languageBox->SetSelection ( INDEX_CHINESE_TRADITIONAL );
-			break;
-		case wxLANGUAGE_UKRAINIAN:
-			languageBox->SetSelection ( INDEX_UKRAINIAN );
-			break;
-		case wxLANGUAGE_SPANISH:
-             languageBox->SetSelection ( INDEX_SPANISH );
-             break;
-		default:
-			languageBox->SetSelection ( INDEX_ENGLISH_US );
-			break;
+		index = languageBox->Append ( wxGetTranslation ( ( **t ).Description ),
+		                              ( void* )( **t ).Language );
+		if (lang == ( **t ).Language)
+			languageBox->SetSelection ( index );
 	}
 
 	libxmlNetAccessBox = new wxCheckBox (
@@ -272,12 +209,8 @@ MyPropertySheet::MyPropertySheet (
 
 	hsizer->Add ( applicationDirEdit, 0, wxALL | wxALIGN_LEFT, 0 );
 	hsizer->Add ( browse, 0, wxLEFT | wxALIGN_LEFT, 5 );
-	hsizerBrowser->Add ( browserCommandEdit, 0, wxALL | wxALIGN_LEFT, 0 );
-	hsizerBrowser->Add ( browseCommand, 0, wxLEFT | wxALIGN_LEFT, 5 );
 	vsizer->Add ( label, 0, wxLEFT | wxTOP | wxALIGN_LEFT, 5 );
 	vsizer->Add ( hsizer, 0, wxALL | wxALIGN_LEFT, 5 );
-	vsizer->Add ( labelBrowser, 0, wxLEFT | wxTOP | wxALIGN_LEFT, 5 );
-	vsizer->Add ( hsizerBrowser, 0, wxALL | wxALIGN_LEFT, 5 );
 
 	vsizer->Add ( labelLanguage, 0, wxLEFT | wxTOP | wxALIGN_LEFT, 5 );
 	vsizer->Add ( languageBox, 0, wxALL | wxALIGN_LEFT, 5 );
@@ -332,7 +265,6 @@ void MyPropertySheet::OnOk ( wxCommandEvent& e )
 	else
 		applicationDir = testDir;
 
-	browserCommand = browserCommandEdit->GetValue(); // permit incorrect value
 	singleInstanceCheck = singleInstanceCheckBox->GetValue();
 	restoreLayout = restoreLayoutBox->GetValue();
 	rememberOpenTabs = rememberOpenTabsBox->GetValue();
@@ -343,48 +275,11 @@ void MyPropertySheet::OnOk ( wxCommandEvent& e )
 	showFullPathOnFrame = fullPathBox->GetValue();
 
 	int languageChoice = languageBox->GetSelection();
-	switch ( languageChoice )
-	{
-        case INDEX_CATALAN:
-            lang = wxLANGUAGE_CATALAN;
-            break;
-		case INDEX_DUTCH:
-            lang = wxLANGUAGE_DUTCH;
-            break;
-        case INDEX_ITALIAN:
-			lang = wxLANGUAGE_ITALIAN;
-			break;
-		case INDEX_GERMAN:
-			lang = wxLANGUAGE_GERMAN;
-			break;
-		case INDEX_CHINESE_SIMPLIFIED:
-			lang = wxLANGUAGE_CHINESE_SIMPLIFIED;
-			break;
-		case INDEX_CHINESE_TRADITIONAL:
-			lang = wxLANGUAGE_CHINESE_TRADITIONAL;
-			break;
-		case INDEX_RUSSIAN:
-			lang = wxLANGUAGE_RUSSIAN;
-			break;
-		case INDEX_FRENCH:
-			lang = wxLANGUAGE_FRENCH;
-			break;
-		case INDEX_SLOVAK:
-			lang = wxLANGUAGE_SLOVAK;
-			break;
-		case INDEX_SPANISH:
-             lang = wxLANGUAGE_SPANISH;
-             break;
-		case INDEX_SWEDISH:
-			lang = wxLANGUAGE_SWEDISH;
-			break;
-		case INDEX_UKRAINIAN:
-			lang = wxLANGUAGE_UKRAINIAN;
-			break;
-		default:
-			lang = wxLANGUAGE_ENGLISH_US;
-			break;
-	}
+	if ( languageChoice != wxNOT_FOUND )
+		lang = (int)languageBox->GetClientData(languageChoice);
+	else
+		lang = wxLANGUAGE_ENGLISH_US;
+
 	e.Skip();
 }
 
@@ -395,13 +290,6 @@ void MyPropertySheet::OnApplicationDirBrowse ( wxCommandEvent& e )
 		applicationDirEdit->SetValue ( browseDialog->GetPath() );
 }
 
-void MyPropertySheet::OnBrowserCommandBrowse ( wxCommandEvent& e )
-{
-	wxFileDialog *browseDialog = new wxFileDialog ( this );
-	if ( browseDialog->ShowModal() == wxID_OK )
-		browserCommandEdit->SetValue ( browseDialog->GetPath() );
-}
-
 XmlCtrlProperties MyPropertySheet::getProperties()
 {
 	return properties;
@@ -412,11 +300,6 @@ wxString MyPropertySheet::getApplicationDir()
 	return applicationDir;
 }
 
-wxString MyPropertySheet::getBrowserCommand()
-{
-	return browserCommand;
-}
-
 bool MyPropertySheet::getSingleInstanceCheck()
 {
 	return singleInstanceCheck;
