@@ -22,9 +22,6 @@
 #	include <enchant.h>
 #else
 #	include "aspell.h"
-#	ifdef __WXMSW__
-#		include "aspellpaths.h"
-#	endif
 #endif
 #include "styledialog.h"
 #include "nocasecompare.h"
@@ -89,7 +86,6 @@ StyleDialog::StyleDialog (
     const wxString& fileNameParameter,
     const wxString& ruleSetDirectoryParameter,
     const wxString& filterDirectoryParameter,
-    const wxString& browserParameter,
     const wxString& ruleSetPresetParameter,
     const wxString& filterPresetParameter,
 #if !defined(USE_ENCHANT) && defined(__WXMSW__)
@@ -108,16 +104,15 @@ StyleDialog::StyleDialog (
 		    size,
 		    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX ),
 		bufferUtf8 ( bufferParameterUtf8 ),
-		fileName ( fileNameParameter ),
-		ruleSetDirectory ( ruleSetDirectoryParameter ),
-		filterDirectory ( filterDirectoryParameter ),
-		browser ( browserParameter ),
-		ruleSetPreset ( ruleSetPresetParameter ),
-		filterPreset ( filterPresetParameter ),
 #if !defined(USE_ENCHANT) && defined(__WXMSW__)
 		aspellDataPath ( aspellDataPathParameter ),
 		aspellDictPath ( aspellDictPathParameter ),
 #endif
+		fileName ( fileNameParameter ),
+		ruleSetDirectory ( ruleSetDirectoryParameter ),
+		filterDirectory ( filterDirectoryParameter ),
+		ruleSetPreset ( ruleSetPresetParameter ),
+		filterPreset ( filterPresetParameter ),
 		type(typeParameter),
 		readOnly ( readOnlyParameter )
 {
@@ -243,7 +238,7 @@ StyleDialog::StyleDialog (
 	reportTopSizer->Add ( comboSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
 	reportTopSizer->Add ( table, 1, wxEXPAND | wxALL, 5 );
 	reportTopSizer->Add ( reportButtonSizer, 0, wxALL, 5 );
-	reportTopSizer->Add ( status, 0 );
+	reportTopSizer->Add ( status, 0, wxEXPAND | wxALL );
 	this->SetSizer ( reportTopSizer );
 
 	createReportButton->SetFocus();
@@ -459,27 +454,21 @@ void StyleDialog::OnReport ( wxCommandEvent& event )
 	ruleSet.Replace ( _(" "), _T("_") );
 	filter = filterPreset + _T ( ".xml" );
 
-	std::string ruleSetDirectoryUtf8,
-	ruleSetUtf8,
-	filterDirectoryUtf8,
-	filterUtf8,
-	pathSeparatorUtf8;
-	ruleSetDirectoryUtf8 = ruleSetDirectory.mb_str ( wxConvUTF8 );
-	ruleSetUtf8 = ruleSet.mb_str ( wxConvUTF8 );
-	filterDirectoryUtf8 = filterDirectory.mb_str ( wxConvUTF8 );
-	filterUtf8 = filter.mb_str ( wxConvUTF8 );
-
+	std::string ruleSetDirectoryLocal ( ruleSetDirectory.mb_str ( wxConvLocal ) );
+	std::string ruleSetUtf8 ( ruleSet.mb_str ( wxConvUTF8 ) );
+	std::string filterDirectoryLocal ( filterDirectory.mb_str ( wxConvLocal ) );
+	std::string filterUtf8 ( filter.mb_str ( wxConvUTF8 ) );
 	wxString separator = wxFileName::GetPathSeparator();
-	pathSeparatorUtf8 = separator.mb_str ( wxConvUTF8 );
+	std::string pathSeparatorLocal ( separator.mb_str ( wxConvLocal ) );
 
 	std::auto_ptr<HouseStyle> hs ( new HouseStyle (
 					   (type == ID_TYPE_SPELL) ? HS_TYPE_SPELL : HS_TYPE_STYLE,
 	                                   bufferUtf8,
-	                                   ruleSetDirectoryUtf8,
+	                                   ruleSetDirectoryLocal,
 	                                   ruleSetUtf8,
-	                                   filterDirectoryUtf8,
+	                                   filterDirectoryLocal,
 	                                   filterUtf8,
-	                                   pathSeparatorUtf8,
+	                                   pathSeparatorLocal,
  #ifdef __WXMSW__
                                        aspellDataPath,
                                        aspellDictPath,
@@ -684,8 +673,7 @@ void StyleDialog::OnStyleWebReport ( wxCommandEvent& event )
 	if ( !wxFileName::FileExists ( tempNameWide ) )
 		return;
 
-	wxString cmd = browser + _T ( " \"" ) + tempNameWide + _T ( "\"" );
-	wxExecute ( cmd, wxEXEC_SYNC );
+	wxLaunchDefaultBrowser ( tempNameWide );
 }
 
 void StyleDialog::OnStyleWebSummary ( wxCommandEvent& event )
@@ -751,8 +739,7 @@ void StyleDialog::OnStyleWebSummary ( wxCommandEvent& event )
 	if ( !wxFileName::FileExists ( tempNameWide ) )
 		return;
 
-	wxString cmd = browser + _T ( " \"" ) + tempNameWide + _T ( "\"" );
-	wxExecute ( cmd, wxEXEC_SYNC );
+	wxLaunchDefaultBrowser ( tempNameWide );
 }
 
 void StyleDialog::styleSetIgnoreAll ( bool ignore )
