@@ -23,22 +23,42 @@
 #include <string>
 #include <vector>
 #include "contexthandler.h"
+#if !defined(USE_ENCHANT)
 #include "aspell.h"
+#else
+namespace enchant
+{
+	class Broker;
+	class Dict;
+}
+#endif
 
 class WrapAspell
 {
 	public:
-		WrapAspell ( std::string lang = "en_US" );
+		WrapAspell (
+                   std::string lang// = "en_US",
+#if !defined(USE_ENCHANT) && defined(__WXMSW__)
+                   , const std::string& aspellDataPathParameter,
+                   const std::string& aspellDictPath
+#endif
+                   );
 		~WrapAspell();
-		inline bool checkWord ( std::string &s );
+		inline bool checkWord ( const std::string &s );
 		void checkString (
 		    std::string &s,
 		    std::vector<ContextMatch> &v,
 		    int contextRange );
 		std::string getSuggestion ( std::string &s );
+		std::string getVersion();
 	private:
+#ifdef USE_ENCHANT
+		enchant::Broker *spell_broker;
+		enchant::Dict *spell_checker;
+#else
 		AspellConfig *spell_config;
 		AspellSpeller *spell_checker;
-		bool checkWord ( char *s, size_t len );
+#endif
+		bool checkWord ( const char *s, size_t len );
 };
 #endif
