@@ -11,7 +11,9 @@
 #include "xmlcopyimg.h"
 #include "binaryfile.h"
 #include "replace.h"
+#include "wrapregex.h"
 #include "mp3album.h"
+//#include "playlistrenamer.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/ole/automtn.h>
@@ -64,6 +66,7 @@ bool WrapDaisy::run (
     bool suppressOptional,
     bool epub,
     bool rtf,
+    bool doc,
     bool fullDaisy,
     bool mp3Album )
 {
@@ -107,7 +110,14 @@ bool WrapDaisy::run (
     if ( !stdStylesheet.empty() ) // stylesheet found
     {
         // #1: convert to canonical XHTML
-        pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+    	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+    	pd->ProcessPendingEvents();
+#endif
+        while (wxTheApp->Pending())
+            wxTheApp->Dispatch();
+
         if ( !pd->Update ( 10, _T("Preparing canonical XHTML...") ) )
         {
             error = _T( "Cancelled" );
@@ -146,7 +156,13 @@ bool WrapDaisy::run (
     
     if ( suppressOptional )
     {
-        pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+    	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+    	pd->ProcessPendingEvents();
+#endif
+        while (wxTheApp->Pending())
+            wxTheApp->Dispatch();
         if ( !pd->Update ( 15, _("Suppressing optional production notes...") ) )
         {
             error = _ ( "Cancelled" );
@@ -157,8 +173,7 @@ bool WrapDaisy::run (
     	if ( !xsp->parse ( output.c_str() ) )
     	{
             frame->newDocument ( output );
-            std::string stdError = xsp->getLastError();
-            error = wxString ( stdError.c_str(), wxConvUTF8, stdError.size() );
+            error = xsp->getLastError();
             return false;
         }
         output = xsp->getBuffer();
@@ -167,7 +182,13 @@ bool WrapDaisy::run (
     if ( quiet )
     {
         // #1.5: apply quiet setting if req'd
-        pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+    	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+    	pd->ProcessPendingEvents();
+#endif
+        while (wxTheApp->Pending())
+            wxTheApp->Dispatch();
         if ( !pd->Update ( 20, _("De-emphasizing production notes...") ) )
         {
             error = _ ( "Cancelled" );
@@ -178,8 +199,7 @@ bool WrapDaisy::run (
     	if ( !xp->parse ( output.c_str() ) )
     	{
             frame->newDocument ( output );
-            std::string stdError = xp->getLastError();
-            error = wxString ( stdError.c_str(), wxConvUTF8, stdError.size() );
+            error = xp->getLastError();
             return false;
         }
         output = xp->getBuffer();
@@ -187,6 +207,19 @@ bool WrapDaisy::run (
 
     // prevent MIME type errors in href="www..." attributes
     Replace::run ( output, "href=\"www", "href=\"http://www", true );
+    
+    // remove em-space
+    Replace::run ( output, "\xE2\x80\x83", " ", true );
+    
+    // remove blank paragraphs
+    Replace::run ( output, "<p></p>", "", true );
+    
+    int replaceCount;
+    WrapRegex regexParaWhitespace ( "<p>\\w+</p>", true );
+    output = regexParaWhitespace.replaceGlobal ( output, &replaceCount );
+    
+    WrapRegex regexContiguousWhitespace ( "[\\t ]+", true, " " );
+    output = regexContiguousWhitespace.replaceGlobal ( output, &replaceCount );
 
     // copy images
     wxString htmlDir, imagesDir, mediaDir;
@@ -234,8 +267,15 @@ bool WrapDaisy::run (
     }
         
     // copy images
-    pd->ProcessPendingEvents();
-    if ( !pd->Update ( 25, _("Copying images and audio files...") ) )
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
+    if ( !pd->Update ( 25, _("Copying files...") ) )
     {
         error = _ ( "Cancelled" );
         return false;
@@ -247,8 +287,7 @@ bool WrapDaisy::run (
 	if ( !xci->parse ( output.c_str() ) )
 	{
         frame->newDocument ( output );
-        std::string stdError = xci->getLastError();
-        error = wxString ( stdError.c_str(), wxConvUTF8, stdError.size() );
+        error = xci->getLastError();
         return false;
     }
     output = xci->getBuffer();
@@ -259,17 +298,24 @@ bool WrapDaisy::run (
     std::ofstream canonicalStream ( stdCanonicalFile.c_str() );//canonicalFile.name().c_str() );
     if ( !canonicalStream )
     {
-        error = _T( "Cannot write canonical XHTML file" );
+        error = _ ( "Cannot write canonical XHTML file" );
         return false;   
     }
     canonicalStream << output.c_str() << std::endl;
     canonicalStream.close();
     
     // #2: convert to DTBook
-    pd->ProcessPendingEvents();
-    if ( !pd->Update ( 40, _T("Preparing DTBook...") ) )
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
+    if ( !pd->Update ( 40, _ ("Preparing DTBook...") ) )
     {
-        error = _T ( "Cancelled" );
+        error = _ ( "Cancelled" );
         return false;   
     }
 
@@ -286,7 +332,7 @@ bool WrapDaisy::run (
     wxString cmd = baseCmd +
         xhtml2dtbookScript +
         _T(" --inputFile=\"") +
-        canonicalFile + //canonicalFile.wideName() +
+        canonicalFile +
         _T("\" --outputFile=\"") +
         dtbFilePath + _T("\"");
 
@@ -315,156 +361,206 @@ bool WrapDaisy::run (
     if ( !error.empty() )
         return false;
         
-    // #2.5: create EPUB version
-    pd->ProcessPendingEvents();
-    if ( !pd->Update ( 50, _T("Preparing ePub...") ) )
+    // #2.5: create ePub version
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
+    if ( epub )
     {
-        error = _T ( "Cancelled" );
-        return false;   
-    }
-
-    wxString epubScript;
+        if ( !pd->Update ( 50, _T("Preparing ePub...") ) )
+        {
+            error = _T ( "Cancelled" );
+            return false;   
+        }
     
-    epubScript += _T("scripts");
-    epubScript += wxFileName::GetPathSeparator();
-    epubScript += _T("create_distribute");
-    epubScript += wxFileName::GetPathSeparator();
-    epubScript += _T("epub");
-    epubScript += wxFileName::GetPathSeparator();
-    epubScript += _T("OPSCreator.taskScript");
-
-    cmd = baseCmd +
-        _T("\"") + epubScript + _T("\" --input=\"") +
-        canonicalFile + //canonicalFile.wideName() +
-        _T("\" --output=\"") +
-        folder + wxFileName::GetPathSeparator() + _T("ebook.epub\"");
+        wxString epubScript;
         
-    result = wxExecute ( cmd, out, err );
-	
-    count = err.GetCount();
-    if ( count )
-    {
-        for ( int i = 0; i < count; i++ )
-        {
-            error += err.Item ( i );
-            error += _T(" ");
-        }
-    }
+        epubScript += _T("scripts");
+        epubScript += wxFileName::GetPathSeparator();
+        epubScript += _T("create_distribute");
+        epubScript += wxFileName::GetPathSeparator();
+        epubScript += _T("epub");
+        epubScript += wxFileName::GetPathSeparator();
+        epubScript += _T("OPSCreator.taskScript");
     
-/*
-    count = out.GetCount();
-    if ( count )
-    {
-        for ( int i = 0; i < count; i++ )
+        cmd = baseCmd +
+            _T("\"") + epubScript + _T("\" --input=\"") +
+            canonicalFile + //canonicalFile.wideName() +
+            _T("\" --output=\"") +
+            folder + wxFileName::GetPathSeparator() + _T("ebook.epub\"");
+            
+        result = wxExecute ( cmd, out, err );
+    	
+        count = err.GetCount();
+        if ( count )
         {
-            error += out.Item ( i );
-            error += _T(" ");
+            for ( int i = 0; i < count; i++ )
+            {
+                error += err.Item ( i );
+                error += _T(" ");
+            }
         }
+        
+    /*
+        count = out.GetCount();
+        if ( count )
+        {
+            for ( int i = 0; i < count; i++ )
+            {
+                error += out.Item ( i );
+                error += _T(" ");
+            }
+        }
+    */
+    
+        if ( !error.empty() )
+            return false;
     }
-*/
-
-    if ( !error.empty() )
-        return false;
 
     // #2.9: convert to RTF
-    pd->ProcessPendingEvents();
-    if ( !pd->Update ( 60, _T("Preparing RTF...") ) )
+    if ( rtf || doc )
     {
-        error = _T ( "Cancelled" );
-        return false;   
-    }
-
-    wxString rtfScript;
-    
-    rtfScript += _T("scripts");
-    rtfScript += wxFileName::GetPathSeparator();
-    rtfScript += _T("create_distribute");
-    rtfScript += wxFileName::GetPathSeparator();
-    rtfScript += _T("text");
-    rtfScript += wxFileName::GetPathSeparator();
-    rtfScript += _T("DtbookToRtf.taskScript");
-
-    wxString rtfFile = folder + wxFileName::GetPathSeparator() + _T("document.rtf");
-    
-    cmd = baseCmd +
-        _T("\"") + rtfScript + _T("\" --input=\"") +
-        dtbFilePath + //dtbFile.wideName() +
-        _T("\" --output=\"") + rtfFile + 
-        _T("\" --inclTOC=\"true\" --inclPagenum=\"false\"");
-        
-    result = wxExecute ( cmd, out, err );
-	
-    count = err.GetCount();
-    if ( count )
-    {
-        for ( int i = 0; i < count; i++ )
-        {
-            error += err.Item ( i );
-            error += _T(" ");
-        }
-    }
-    
-/*
-    count = out.GetCount();
-    if ( count )
-    {
-        for ( int i = 0; i < count; i++ )
-        {
-            error += out.Item ( i );
-            error += _T(" ");
-        }
-    }
-*/
-
-    if ( !error.empty() )
-        return false;    
-
-    // #2.9.5: convert to binary Word
-    // (Win only; otherwise create copy with *.doc extension)
-
-    pd->ProcessPendingEvents();
-    if ( !pd->Update ( 60, _T("Preparing Word document...") ) )
-    {
-        error = _T ( "Cancelled" );
-        return false;   
-    }
-
-
-    wxString docFile = rtfFile;
-    docFile.Replace ( _T(".rtf"), _T(".doc") );
-
-#ifdef __WXMSW__
-    wxAutomationObject wordObject, documentObject;
-
-    if ( wordObject.CreateInstance ( _T("Word.Application") ) )
-    {
-        wxVariant openParams[2];
-        openParams[0] = rtfFile;
-        openParams[1] = false;   
-
-        wordObject.CallMethod(_("documents.open"), 2, openParams);
-        if (!wordObject.GetObject(documentObject, _("ActiveDocument"))) 
-        { 
-            error = _("Cannot open ") + rtfFile;
-            return false;
-        }
-        wxVariant saveAsParams[2];
-        saveAsParams[0] = docFile;
-        saveAsParams[1] = (long)0; //wdFormatDocument
-        if ( !documentObject.CallMethod(_("SaveAs"), 2, saveAsParams) )
-        {
-            //error = _("Cannot save ") + docFile;
-            //return false;
-        }
-        documentObject.CallMethod(_("Close"), 0, NULL );
-        wordObject.CallMethod(_T("Quit"), 0, NULL );
-    }
+#if wxCHECK_VERSION(2,9,0)
+    	pd->GetEventHandler()->ProcessPendingEvents();
 #else
-    wxCopyFile ( rtfFile, docFile );
+   		pd->ProcessPendingEvents();
 #endif
-
+        while (wxTheApp->Pending())
+            wxTheApp->Dispatch();
+    
+        if ( !pd->Update ( 60, _T("Preparing RTF...") ) )
+        {
+            error = _T ( "Cancelled" );
+            return false;   
+        }
+    
+        wxString rtfScript;
+        
+        rtfScript += _T("scripts");
+        rtfScript += wxFileName::GetPathSeparator();
+        rtfScript += _T("create_distribute");
+        rtfScript += wxFileName::GetPathSeparator();
+        rtfScript += _T("text");
+        rtfScript += wxFileName::GetPathSeparator();
+        rtfScript += _T("DtbookToRtf.taskScript");
+    
+        wxString rtfFile, tempRtfFile, docFile, tempDocFile;
+        rtfFile = folder + wxFileName::GetPathSeparator() + _T("document.rtf");
+        tempRtfFile = folder + wxFileName::GetPathSeparator() + _T("html") +
+            wxFileName::GetPathSeparator() + _T("document.rtf");
+     
+        docFile = rtfFile;
+        tempDocFile = tempRtfFile;
+        docFile.Replace ( _T(".rtf"), _T(".doc") );
+        tempDocFile.Replace ( _T(".rtf"), _T(".doc") );
+        
+        cmd = baseCmd +
+            _T("\"") + rtfScript + _T("\" --input=\"") +
+            dtbFilePath + //dtbFile.wideName() +
+            _T("\" --output=\"") + rtfFile + 
+            _T("\" --inclTOC=\"true\" --inclPagenum=\"false\"");
+            
+        result = wxExecute ( cmd, out, err );
+    	
+        count = err.GetCount();
+        if ( count )
+        {
+            for ( int i = 0; i < count; i++ )
+            {
+                error += err.Item ( i );
+                error += _T(" ");
+            }
+        }
+        
+    /*
+        count = out.GetCount();
+        if ( count )
+        {
+            for ( int i = 0; i < count; i++ )
+            {
+                error += out.Item ( i );
+                error += _T(" ");
+            }
+        }
+    */
+    
+        if ( !error.empty() )
+            return false;    
+    
+        // #2.9.5: convert to binary Word
+        // (Win only; otherwise create copy with *.doc extension)
+    
+#if wxCHECK_VERSION(2,9,0)
+        pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   		pd->ProcessPendingEvents();
+#endif
+        while (wxTheApp->Pending())
+            wxTheApp->Dispatch();
+        
+        if ( !pd->Update ( 60, _T("Preparing Word document...") ) )
+        {
+            error = _T ( "Cancelled" );
+            return false;   
+        }
+    
+        //wxString docFile = rtfFile;
+        //docFile.Replace ( _T(".rtf"), _T(".doc") );
+    
+    #ifdef __WXMSW__
+        wxAutomationObject wordObject, documentObject;
+    
+        if ( wordObject.CreateInstance ( _T("Word.Application") ) )
+        {
+            wxVariant openParams[2];
+            openParams[0] = rtfFile;//tempRtfFile
+            openParams[1] = false;   
+    
+            wordObject.CallMethod(_("documents.open"), 2, openParams);
+            if (!wordObject.GetObject(documentObject, _("ActiveDocument"))) 
+            { 
+                error = _("Cannot open ") + rtfFile;//tempRtfFile;
+                return false;
+            }
+            wxVariant saveAsParams[2];
+            saveAsParams[0] = docFile;//tempDocFile;//
+            saveAsParams[1] = (long)0; //wdFormatDocument
+            if ( !documentObject.CallMethod(_("SaveAs"), 2, saveAsParams) )
+            {
+                //error = _("Cannot save ") + docFile;
+                //return false;
+            }
+            documentObject.CallMethod(_("Close"), 0, NULL );
+            wordObject.CallMethod(_T("Quit"), 0, NULL );
+        }
+    #else
+        //wxCopyFile ( tempRtfFile, tempDocFile );
+        wxCopyFile ( rtfFile, docFile );
+    #endif
+    
+        //wxCopyFile ( tempRtfFile, rtfFile );
+        //wxCopyFile ( tempDocFile, docFile );
+        //wxRemoveFile ( tempRtfFile );
+        //wxRemoveFile ( tempDocFile );
+    }
+    
     // #3: convert to full DAISY book
-    pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
+    if ( !fullDaisy )
+        return true; // no full DAISY, no audio
+        
     if ( !pd->Update ( 70, _T("Preparing DAISY books...") ) )
     {
         error = _T ( "Cancelled" );
@@ -516,8 +612,18 @@ bool WrapDaisy::run (
     if ( !error.empty() )
         return false;
 
+    if ( !mp3Album )
+        return true;
+
     // #4: create MP3 album
-    pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
     if ( !pd->Update ( 80, _T("Preparing MP3 album...") ) )
     {
         error = _T ( "Cancelled" );
@@ -591,19 +697,27 @@ bool WrapDaisy::run (
         return false;
         
 
-    pd->ProcessPendingEvents();
+#if wxCHECK_VERSION(2,9,0)
+   	pd->GetEventHandler()->ProcessPendingEvents();
+#else
+   	pd->ProcessPendingEvents();
+#endif
+    while (wxTheApp->Pending())
+        wxTheApp->Dispatch();
+
+/*
     if ( !pd->Update ( 90, _T("Updating playlists...") ) )
     {
         error = _T ( "Cancelled" );
         return false;   
     }
 
+
     //rename mp3 playlists
-/*
-    PlayListRenamer plr;
-    std::string stdAlbumDir = ( const char *) albumDir.mb_str ( wxConvUTF8 );
-    plr.run ( stdAlbumDir );
-*/  
+    //albumDir += wxFileName::GetPathSeparator();
+    //PlayListRenamer plr;
+    //std::string stdAlbumDir = ( const char *) albumDir.mb_str ( wxConvUTF8 );
+    //plr.run ( stdAlbumDir );
 
     //rename mp3 files in //z3986/
     wxFileName fn ( dtbFilePath );
@@ -681,6 +795,7 @@ bool WrapDaisy::run (
         wxFileName::GetPathSeparator() +
         _T("cover.jpg");
     wxCopyFile ( albumCover, destAlbumCover, true );
+*/
     
     return true;
 }
