@@ -22,10 +22,24 @@
 #define FILE_FILTER _("All files (*.*)|*.*|XML (*.xml)|*.xml|XHTML (*.html)|*.html|DTD (*.dtd)|*.dtd|XML Schema (*.xsd)|*.xsd|RELAX NG grammar (*.rng)|*.rng|XSL (*.xsl)|*.xsl")
 #else
 #define FILE_FILTER _("All files (*)|*|XML (*.xml)|*.xml|XHTML (*.html)|*.html|DTD (*.dtd)|*.dtd|XML Schema (*.xsd)|*.xsd|RELAX NG grammar (*.rng)|*.rng|XSL (*.xsl)|*.xsl")
-
 #endif
+#if wxCHECK_VERSION(2,9,0)
+#define ABOUT_COPYRIGHT _(_T("Copyright © 2005-2009 Gerald Schmidt <gnschmidt@users.sourceforge.net>"))
+#else
 #define ABOUT_COPYRIGHT _("Copyright © 2005-2009 Gerald Schmidt <gnschmidt@users.sourceforge.net>")
-#define ABOUT_DESCRIPTION _("\nXML Copy Editor is free software released under the GNU\nGeneral Public License.\n\nMany thanks are due to Tim van Niekerk, Matt Smigielski,\nDavid Scholl, Jan Merka, Marcus Bingenheimer, Roberto\nRosselli Del Turco, Ken Zalewski, C.J. Meidlinger,\nThomas Zajic, Viliam Búr, David Håsäther, François\nBadier, Thomas Wenzel, Roger Sperberg, SHiNE CsyFeK,\nHSU PICHAN, YANG SHUFUN, CHENG PAULIAN,\nCHUANG KUO-PING, Justin Dearing, Serhij Dubyk,\nAntonio Angelo, Jose Luis Rivero, Siarhei Kuchuk,\nIan Abbott, Kev James, Marcos Pérez González, Anh\nTrinh, Rob Elemans and Robert Falc\xf3 Miramontes.")
+#endif
+#define ABOUT_DESCRIPTION _("\nXML Copy Editor is free software released under the GNU\nGeneral Public License.\n\nMany thanks are due to ")
+#define ABOUT_CONTRIBUTORS _T(\
+"Tim van Niekerk, Matt Smigielski,\n"\
+"David Scholl, Jan Merka, Marcus Bingenheimer, Roberto\n"\
+"Rosselli Del Turco, Ken Zalewski, C.J. Meidlinger,\n"\
+"Thomas Zajic, Viliam Búr, David Håsäther, François\n"\
+"Badier, Thomas Wenzel, Roger Sperberg, SHiNE CsyFeK,\n"\
+"HSU PICHAN, YANG SHUFUN, CHENG PAULIAN,\n"\
+"CHUANG KUO-PING, Justin Dearing, Serhij Dubyk,\n"\
+"Antonio Angelo, Jose Luis Rivero, Siarhei Kuchuk,\n"\
+"Ian Abbott, Kev James, Marcos Pérez González, Anh Trinh,\n"\
+"Rob Elemans, Robert Falcó Miramontes and Zane U. Ji.")
 #define ABOUT_LICENSE _T(\
 "This program is free software; you can redistribute it\n"\
 "and/or modify it under the terms of the GNU General Public\n"\
@@ -41,7 +55,7 @@
 "License along with this program; if not, write to the Free\n"\
 "Software Foundation, Inc., 59 Temple Place, Suite 330,\n"\
 "Boston, MA 02111-1307 USA.")
-#define ABOUT_VERSION _T("1.2.0.6")
+#define ABOUT_VERSION _T("1.2.0.8")
 #define XMLCE_VAR _T("XMLCE_VAR")
 
 #ifdef __WXMSW__
