@@ -25,6 +25,8 @@
 #include "xmlcopyeditor.h" // needed to enable validation-as-you-type alerts
 #include <utility>
 #include <memory>
+#include "validationthread.h"
+
 
 // adapted from wxSTEdit (c) 2005 John Labenski, Otto Wyss
 #define XMLCTRL_HASBIT(value, bit) (((value) & (bit)) != 0)
@@ -38,6 +40,7 @@ BEGIN_EVENT_TABLE ( XmlCtrl, wxStyledTextCtrl )
 	EVT_LEFT_UP ( XmlCtrl::OnMouseLeftUp )
 	EVT_RIGHT_UP ( XmlCtrl::OnMouseRightUp )
 	EVT_MIDDLE_DOWN ( XmlCtrl::OnMiddleDown )
+	EVT_COMMAND(wxID_ANY, wxEVT_COMMAND_VALIDATION_COMPLETED, XmlCtrl::OnValidationCompleted)
 END_EVENT_TABLE()
 
 // global protection for validation threads
@@ -45,37 +48,31 @@ wxCriticalSection xmlcopyeditorCriticalSection;
 
 XmlCtrl::XmlCtrl (
     wxWindow *parent,
-    XmlCtrlProperties propertiesParameter,
+    const XmlCtrlProperties &propertiesParameter,
     bool *protectTagsParameter,
     int visibilityStateParameter,
     int typeParameter,
     wxWindowID id,
     const char *buffer, // could be NULL
     size_t bufferLen,
-    const std::string& catalogPathParameter,
-    const std::string& catalogUtilityPathParameter,
     const std::string& basePathParameter,
     const std::string& auxPathParameter,
     const wxPoint& position,
     const wxSize& size,
-    long style ) : wxStyledTextCtrl ( parent, id, position, size, style ),
-		type ( typeParameter ),
-		protectTags ( protectTagsParameter ),
-		visibilityState ( visibilityStateParameter ),
-		catalogPath ( catalogPathParameter ),
-		catalogUtilityPath ( catalogUtilityPathParameter ),
-		basePath ( basePathParameter ),
-		auxPath ( auxPathParameter )
+    long style
+    )
+	: wxStyledTextCtrl ( parent, id, position, size, style )
+	, type ( typeParameter )
+	, protectTags ( protectTagsParameter )
+	, visibilityState ( visibilityStateParameter )
+	, basePath ( basePathParameter )
+	, auxPath ( auxPathParameter )
 {
 	validationThread = NULL;
-	validationStarted = false;
-	validationFinished = false;
+
 	grammarFound = false;
 	validationRequired = (buffer) ? true : false; // NULL for plain XML template
 
-	validationReleasePtr = new bool;
-	*validationReleasePtr = false;
-	
 	currentMaxLine = 1;
 
 	applyProperties ( propertiesParameter );
@@ -95,7 +92,11 @@ XmlCtrl::XmlCtrl (
 		bufferLen = strlen ( DEFAULT_XML_DECLARATION_UTF8 );
 	}
 
+#if wxCHECK_VERSION(2,9,0)
+	AddTextRaw ( buffer, bufferLen );
+#else
 	SendMsg ( 2001, bufferLen, ( long ) ( const char * ) buffer );
+#endif
 
 	SetSelection ( 0, 0 );
 
@@ -120,10 +121,10 @@ XmlCtrl::XmlCtrl (
 	applyVisibilityState ( visibilityState );
 	lineBackgroundState = BACKGROUND_STATE_NORMAL;
 
-	for ( int i = 0; i < wxSTC_INDIC_MAX; ++i ) 
+	for ( int i = 0; i < wxSTC_INDIC_MAX; ++i )
 		IndicatorSetStyle ( i, wxSTC_INDIC_HIDDEN );
 	IndicatorSetStyle ( 2, wxSTC_INDIC_SQUIGGLE );
-	IndicatorSetForeground ( 0, *wxRED );
+	IndicatorSetForeground ( 2, *wxRED );
 }
 
 
@@ -133,17 +134,12 @@ XmlCtrl::~XmlCtrl()
 	elementMap.clear();
 	entitySet.clear();
 
-	if ( validationStarted && !validationFinished )
+	if ( validationThread != NULL )
 	{
-		*validationReleasePtr = true;
-
-		// don't delete validationReleasePtr because the thread will check the value before exiting
-		// this means that 1 variable of type bool will be leaked in this case
-		// the alternative is waiting for the validation thread to finish, which can take anything up to several minutes
-		return;
+		validationThread->PendingDelete();
+		//validationThread->Delete();
+		//delete validationThread;
 	}
-	
-	delete validationReleasePtr;
 }
 
 
@@ -169,28 +165,30 @@ void XmlCtrl::OnIdle ( wxIdleEvent& event )
 {
 	if ( properties.number && type != FILE_TYPE_BINARY )
 		adjustNoColumnWidth(); // exits if unchanged
+}
 
-	// poll validation thread output if any
-	
+void XmlCtrl::OnValidationCompleted ( wxCommandEvent &event )
+{
+	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
+
+	if ( validationThread == NULL )
+		return;
+
+	MyFrame *frame = (MyFrame *)GetGrandParent();
+	clearErrorIndicators ( GetLineCount() );
+	if ( validationThread->isSucceeded() )
 	{
-		wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
-		if (validationStarted && validationFinished)
-		{
-			validationStarted = false;
-			MyFrame *frame = (MyFrame *)GetGrandParent();
-			if ( validationSuccess )
-			{
-				clearErrorIndicators ( GetLineCount() );
-				frame->statusProgress ( wxEmptyString );
-			}
-			else
-			{
-				clearErrorIndicators ( GetLineCount() );
-				setErrorIndicator ( validationPosition.first - 1, 0 );
-				frame->statusProgress ( wxString ( validationMessage.c_str(), wxConvUTF8, validationMessage.size() ) );
-			}
-		}
+		frame->statusProgress ( wxEmptyString );
 	}
+	else
+	{
+		setErrorIndicator ( validationThread->getPosition().first - 1, 0 );
+		frame->statusProgress ( validationThread->getMessage() );
+	}
+
+	validationThread->Wait();
+	delete validationThread;
+	validationThread = NULL;
 }
 
 void XmlCtrl::OnChar ( wxKeyEvent& event )
@@ -347,41 +345,6 @@ void XmlCtrl::handleDelete ( wxKeyEvent& event )
 
 	validationRequired = true;
 
-	// unindent range
-	int start, end, startLine, endLine;
-	start = GetSelectionStart();
-	end = GetSelectionEnd();
-	startLine = LineFromPosition ( start );
-	endLine = LineFromPosition ( end );
-	
-	// add test so range unindentation only happens when caret is flush with left margin
-	int column = GetColumn ( GetCurrentPos() );
-	
-	if ( startLine != endLine && !column)
-	{
-		if ( startLine > endLine )
-		{
-			int temp;
-			temp = startLine;
-			startLine = endLine;
-			endLine = temp;
-			temp = start;
-			start = end;
-			end = temp;
-		}
-		if ( GetColumn ( start ) < GetLineIndentation ( startLine )  &&
-		        GetColumn ( end ) < GetLineIndentation ( endLine ) )
-		{
-			for ( int i = startLine; i <= endLine; i++ )
-			{
-				int current = GetLineIndentation ( i );
-				if ( current > 0 )
-					SetLineIndentation ( i, current - 1 );
-			}
-			return;
-		}
-	}
-
 	if ( !canMoveRightAt ( GetCurrentPos() ) &&
 	        GetSelectionStart() == GetSelectionEnd() )
 	{
@@ -512,24 +475,18 @@ void XmlCtrl::handleOpenAngleBracket ( wxKeyEvent& event )
 	if ( parentCloseAngleBracket < 0 )
 		return;
 
-	wxString wideParent = getLastElementName ( parentCloseAngleBracket );
-	if ( wideParent.empty() )
-		return;
-
-	std::string parent = ( const char * ) wideParent.mb_str ( wxConvUTF8 );
-
+	wxString parent = getLastElementName ( parentCloseAngleBracket );
 	if ( elementMap.find ( parent ) == elementMap.end() )
 		return;
 
-	wxString choice, conversion;
-	std::set<std::string> childSet = elementMap[parent];
-	std::set<std::string>::iterator it;
+	wxString choice;
+	std::set<wxString> &childSet = elementMap[parent];
+	std::set<wxString>::iterator it;
 	for ( it = childSet.begin(); it != childSet.end(); it++ )
 	{
 		if ( !choice.empty() )
 			choice.append ( _T ( "<" ) );
-		conversion = wxString ( it->c_str(), wxConvUTF8, it->size() );
-		choice.append ( conversion );
+		choice.append ( *it );
 	}
 	if ( !choice.empty() )
 		UserListShow ( 0, choice );
@@ -555,9 +512,7 @@ void XmlCtrl::handleCloseAngleBracket ( wxKeyEvent& event )
 	wxString elementName = getLastElementName ( pos );
 	if ( !elementName.empty() )
 	{
-		std::map<std::string, std::set<std::string> > map;
-		std::string elementNameUtf8 = ( const char * ) elementName.mb_str ( wxConvUTF8 );
-		attributeMap.insert ( make_pair ( elementNameUtf8, map ) );
+		attributeMap[elementName]; // Just want to put it there
 	}
 
 	// exit condition 1
@@ -606,8 +561,6 @@ void XmlCtrl::handleEquals ( wxKeyEvent& event )
 	if ( AutoCompActive() )
 		AutoCompCancel();
 
-	wxString choice, elementName, attributeName, conversion;
-	std::string elementNameUtf8, attributeNameUtf8;
 	int pos = GetCurrentPos();
 	if ( pos <= 0 || getLexerStyleAt ( pos - 1 ) != wxSTC_H_ATTRIBUTE )
 	{
@@ -618,19 +571,15 @@ void XmlCtrl::handleEquals ( wxKeyEvent& event )
 	SetSelection ( pos + 2, pos + 2 );
 
 	// tbd: identify possible attribute values
+	wxString choice, elementName, attributeName;
 	elementName = getLastElementName ( pos );
 	attributeName = getLastAttributeName ( pos );
 
-	elementNameUtf8 = elementName.mb_str ( wxConvUTF8 );
-	attributeNameUtf8 = attributeName.mb_str ( wxConvUTF8 );
-
-	std::set<std::string> valueSet;
-
-	valueSet = attributeMap[elementNameUtf8][attributeNameUtf8];
+	std::set<wxString> &valueSet = attributeMap[elementName][attributeName];
 	if ( valueSet.empty() )
 		return;
 
-	std::set<std::string>::iterator valueSetIterator;
+	std::set<wxString>::iterator valueSetIterator;
 	int cutoff = BUFSIZ;
 	for ( valueSetIterator = valueSet.begin();
 	        valueSetIterator != valueSet.end();
@@ -640,11 +589,7 @@ void XmlCtrl::handleEquals ( wxKeyEvent& event )
 			break;
 		if ( !choice.empty() )
 			choice.Append ( _T ( "<" ) );
-		conversion = wxString (
-		                 valueSetIterator->c_str(),
-		                 wxConvUTF8,
-		                 valueSetIterator->size() );
-		choice.Append ( conversion );
+		choice.Append ( *valueSetIterator );
 	}
 
 	if ( !choice.empty() )
@@ -656,12 +601,7 @@ void XmlCtrl::handleSpace ( wxKeyEvent& event )
 	if ( AutoCompActive() )
 		AutoCompCancel();
 
-	wxString elementName, choice, conversion;
-	std::string elementNameUtf8;
 	int pos = GetCurrentPos();
-	std::map<std::string, std::set<std::string> > currentAttributeMap;
-	std::map<std::string, std::set<std::string> >::iterator
-	attributeIterator;
 	if ( pos <= 2 )
 	{
 		event.Skip();
@@ -700,30 +640,24 @@ void XmlCtrl::handleSpace ( wxKeyEvent& event )
 	}
 
 	AddText ( _T ( " " ) );
-	elementName = getLastElementName ( pos );
-	wxString tag = GetTextRange ( tagStartPos, pos );
 
-	elementNameUtf8 = elementName.mb_str ( wxConvUTF8 );
-	if ( attributeMap.find ( elementNameUtf8 ) == attributeMap.end() )
+	wxString elementName = getLastElementName ( pos );
+	if ( attributeMap.find ( elementName ) == attributeMap.end() )
 		return;
-	currentAttributeMap = attributeMap[elementNameUtf8];
-	for (
-	    attributeIterator = currentAttributeMap.begin();
-	    attributeIterator != currentAttributeMap.end();
-	    attributeIterator++ )
-	{
-		conversion = wxString (
-		                 attributeIterator->first.c_str(),
-		                 wxConvUTF8,
-		                 attributeIterator->first.size() );
 
+	wxString choice;
+	wxString tag = GetTextRange ( tagStartPos, pos );
+	std::map<wxString, std::set<wxString> > &curAttMap = attributeMap[elementName];
+	std::map<wxString, std::set<wxString> >::iterator it;
+	for ( it = curAttMap.begin(); it != curAttMap.end(); it++ )
+	{
 		// avoid duplicate attributes
-		if ( tag.Contains ( conversion + _T ( "=" ) ) )
+		if ( tag.Contains ( it->first + _T ( "=" ) ) )
 			continue;
 
 		if ( !choice.empty() )
 			choice.Append ( _T ( "<" ) );
-		choice.Append ( conversion );
+		choice.Append ( it->first );
 	}
 	if ( !choice.empty() )
 	{
@@ -755,14 +689,13 @@ void XmlCtrl::handleAmpersand ( wxKeyEvent& event )
 	{
 		AddText ( _T ( "&" ) );
 		wxString choice;
-		std::set<std::string>::iterator it;
-		it = entitySet.begin();
-		choice += wxString ( it->c_str(), wxConvUTF8, it->size() );
+		std::set<wxString>::iterator it = entitySet.begin();
+		choice += *it;
 		choice += _T ( ";" );
 		for ( it++; it != entitySet.end(); it++ )
 		{
 			choice += _T ( "<" );
-			choice += wxString ( it->c_str(), wxConvUTF8, it->size() );
+			choice += *it;
 			choice += _T ( ";" );
 		}
 		UserListShow ( 0, choice );
@@ -1002,26 +935,13 @@ wxString XmlCtrl::getLastElementName ( int pos )
 	return GetTextRange ( startPos, iteratorPos );
 }
 
-std::set<wxString> XmlCtrl::getChildren ( const wxString& parent )
+const std::set<wxString> &XmlCtrl::getChildren ( const wxString& parent )
 {
-	std::string parentUtf8 = ( const char * ) parent.mb_str ( wxConvUTF8 );
-	std::set<wxString> mySet;
-
-	if ( elementMap.find ( parentUtf8 ) == elementMap.end() )
+	static std::set<wxString> mySet;
+	if ( elementMap.find ( parent ) == elementMap.end() )
 		return mySet;
 
-	std::set<std::string> myUtf8Set = elementMap[parentUtf8];
-	std::string currentUtf8;
-	wxString currentWide;
-
-	std::set<std::string>::iterator it;
-	for ( it = myUtf8Set.begin(); it != myUtf8Set.end(); it++ )
-	{
-		currentUtf8 = *it;
-		currentWide = wxString ( currentUtf8.c_str(), wxConvUTF8, currentUtf8.size() );
-		mySet.insert ( currentWide );
-	}
-	return mySet;
+	return elementMap[parent];
 }
 
 wxString XmlCtrl::getLastAttributeName ( int pos )
@@ -1120,7 +1040,6 @@ void XmlCtrl::updatePromptMaps ( const char *buffer, size_t bufferLen )
 	elementMap.clear();
 	elementStructureMap.clear();
 	std::auto_ptr<XmlPromptGenerator> xpg ( new XmlPromptGenerator (
-	                                            catalogPath,
 	                                            basePath,
 	                                            auxPath ) );
 	xpg->parse ( buffer, bufferLen );
@@ -1130,15 +1049,15 @@ void XmlCtrl::updatePromptMaps ( const char *buffer, size_t bufferLen )
 	xpg->getElementStructureMap ( elementStructureMap );
 	xpg->getEntitySet ( entitySet );
 	grammarFound = xpg->getGrammarFound();
-	entitySet.insert ( "amp" );
-	entitySet.insert ( "apos" );
-	entitySet.insert ( "quot" );
-	entitySet.insert ( "lt" );
-	entitySet.insert ( "gt" );
+	entitySet.insert ( _T ( "amp" ) );
+	entitySet.insert ( _T ( "apos" ) );
+	entitySet.insert ( _T ( "quot" ) );
+	entitySet.insert ( _T ( "lt" ) );
+	entitySet.insert ( _T ( "gt" ) );
 }
 
 void XmlCtrl::applyProperties (
-    XmlCtrlProperties propertiesParameter,
+    const XmlCtrlProperties &propertiesParameter,
     bool zoomOnly )
 {
 	properties = propertiesParameter;
@@ -1858,16 +1777,15 @@ wxString XmlCtrl::getOpenTag ( const wxString& element )
 {
 	wxString openTag;
 	openTag = _T ( "<" ) + element;
-	std::set<std::string> requiredAttributeSet;
-	std::set<std::string>::iterator it;
-	std::string key = ( const char * ) element.mb_str ( wxConvUTF8 );
-	requiredAttributeSet = requiredAttributeMap[key];
+	std::set<wxString> requiredAttributeSet;
+	std::set<wxString>::iterator it;
+	requiredAttributeSet = requiredAttributeMap[element];
 	if ( !requiredAttributeSet.empty() )
 	{
 		for ( it = requiredAttributeSet.begin(); it != requiredAttributeSet.end(); it++ )
 		{
 			openTag += _T ( " " );
-			openTag += wxString ( it->c_str(), wxConvUTF8, it->size() );
+			openTag += *it;
 			openTag += _T ( "=\"\"" );
 		}
 	}
@@ -2005,31 +1923,24 @@ void XmlCtrl::toggleLineBackground()
 	SetCaretLineBackground ( ( lineBackgroundState == BACKGROUND_STATE_NORMAL ) ? baseBackground : alternateBackground );
 }
 
-std::set<std::string> XmlCtrl::getEntitySet()
+const std::set<wxString> &XmlCtrl::getEntitySet()
 {
 	return entitySet;
 }
 
-std::set<std::string> XmlCtrl::getAttributes ( const wxString& parent )
+const std::set<std::string> &XmlCtrl::getAttributes ( const wxString& parent )
 {
-	std::set<std::string> retVal;
+	static std::set<std::string> retVal;
 	return retVal;
 }
 
-std::string XmlCtrl::getElementStructure ( const wxString& element )
+wxString XmlCtrl::getElementStructure ( const wxString& element )
 {
-	std::string stdElement, ret;
-	stdElement = element.mb_str ( wxConvUTF8);
-	
-	if ( elementStructureMap.find ( stdElement ) == elementStructureMap.end() )
+	if ( elementStructureMap.find ( element ) == elementStructureMap.end() )
 	{
-		ret = "";
+		return wxEmptyString;
 	}
-	else
-	{
-		ret = elementStructureMap[stdElement];
-	}
-	return ret;
+	return elementStructureMap[element];
 }
 
 bool XmlCtrl::backgroundValidate()
@@ -2055,50 +1966,35 @@ bool XmlCtrl::backgroundValidate (
 {
 	if ( !validationRequired )
 		return true;
-	
+
 	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
-	if ( validationStarted && !validationFinished )
+
+	if ( validationThread != NULL )
 	{
-		*validationReleasePtr = true;
 		return true; // wait for next idle cycle call from main app frame
 	}
 	validationRequired = false;
-	
-	*validationReleasePtr = false;
+
 	validationThread = new ValidationThread(
+		GetEventHandler(),
 		buffer,
-		system,
-		catalogPath.c_str(),
-		catalogUtilityPath.c_str(),
-		&validationFinished,
-		&validationSuccess,
-		validationReleasePtr,
-		&validationPosition,
-		&validationMessage
+		system
 	);
 
-	if ( validationThread->Create() != wxTHREAD_NO_ERROR )
+	if ( validationThread->Create() != wxTHREAD_NO_ERROR
+	    || validationThread->Run() != wxTHREAD_NO_ERROR )
 	{
-		validationStarted = false;
-		validationFinished = true;
+		delete validationThread;
+		validationThread = NULL;
 		return false;
 	}
-	
-	validationStarted = true;
-	validationFinished = false;
-	validationThread->Run();
+
 	return true;
 }
 
 std::string XmlCtrl::myGetTextRaw()
 {
-	int len  = GetTextLength();
-	char *buf = new char[len+1];
-	SendMsg ( 2182, len+1, ( long ) buf );
-	buf[len] = '\0';
-	std::string ret ( buf );
-	delete[] buf;
-	return ret;
+	return ( const char * ) GetTextRaw();
 }
 
 void XmlCtrl::setErrorIndicator ( int line, int column )
@@ -2121,14 +2017,15 @@ void XmlCtrl::clearErrorIndicators ( int maxLine )
 	if ( maxLine < 0 )
 		return;
 
-	int documentLength = GetLength();
-	if ( !documentLength )
+	int length = GetLength();
+	if ( !length )
 		return;
 
 	StartStyling ( 0, wxSTC_INDIC2_MASK );
 
-	int length;
-	length = ( maxLine ) ? GetLineEndPosition ( maxLine ) : documentLength;
+	int end = GetEndStyled();
+	length = ( maxLine ) ? GetLineEndPosition ( maxLine ) : length;
+	if ( length > end ) length = end;
 	SetStyling ( length, 0 );
 }
 
